#!/usr/bin/python3
# -*- coding: utf-8 -*-

__author__ = 'wenbo'

import os
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import QLabel, QWidget, QDialog, QLineEdit

from mainui.switchbutton import SwitchBtn
from utils.enums import AUTOSTART, CRONDAILY, CONF_MODEL_PATH, LAST_START_TIME
from mainui.systembutton import *
from mainui.titlebar import TitleBar
import  configparser
import gettext
from xdg import BaseDirectory as xdg
import shutil
gettext.textdomain("kylin-update-manager")

_ = gettext.gettext

class ConfigWidget(QDialog):
    #
    # app_msg = None
    # size_change = pyqtSignal(str, bool)
    m_parent = None
    bootauto_check_file = pyqtSignal(bool)
    daily_check_file = pyqtSignal(bool)

    conf_path = os.path.join(xdg.xdg_cache_home, "kylin-update-manager/kylin-update-manager.conf")
    def __init__(self, parent=None):
        # QWidget.__init__(self,parent)

        super(ConfigWidget, self).__init__(parent)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Tool)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setWindowTitle(_("Kylin Update Manager"))
        # self.setStyleSheet("QWidget{background-color}")
        self.setFixedSize(420, 320)
        self.change_tip = False
        self.parent = parent
        self.mouse_press = False
        palette = QPalette()
        palette.setColor(QPalette.Background, QColor(255, 255, 255))
        self.setPalette(palette)
        self.ui_init()
        self.stylesheet_init()

    #
    # 函数：初始化ui
    #
    def ui_init(self):
        self.setting_label=QLabel(self)
        self.setting_label.setFixedWidth(100)


        self.close_btn=SystemButton(False,False,True,self)
        self.close_btn.setGeometry(376, 14, 30, 30)
        self.close_btn.clicked.connect(self.slot_click_close_button)

        self.btvalue_init()
        self.bootautocheckbt = SwitchBtn(self,initcheck=self.bootautocheckbt_initchecked) # 开机自动检测按钮
        self.dailycheckbt = SwitchBtn(self,initcheck=self.dailycheckbt_initchecked) # 每日定时检测按钮

        self.bootautochecklabel = QLabel(self)
        self.dailycheclabel = QLabel(self)
        self.changeiplabel = QLabel(self)
        self.ipedit = QLineEdit(self)
        self.checkBtn = QPushButton(self)
        self.init_connect()
    #
    # 函数：初始化样式
    #
    def stylesheet_init(self):
        self.setting_label.setStyleSheet("QLabel{font-size:24px;color:#444444;}")
        self.setting_label.setText(_("Setting"))
        self.setting_label.move(40,60)

        self.bootautochecklabel.setStyleSheet("QLabel{font-size:14px;color:#444444;}")
        self.bootautochecklabel.setText(_("Auto start"))
        self.bootautochecklabel.move(90,130)
        #self.bootautochecklabel.setGeometry(80,120,150,15)

        self.dailycheclabel.setStyleSheet("QLabel{font-size:14px;color:#444444;}")
        self.dailycheclabel.setText(_("Timing start"))
        self.dailycheclabel.setGeometry(90,180,150,15)

        self.bootautocheckbt.setGeometry(250,130,50,24)

        self.dailycheckbt.setGeometry(250,180,50,24)
        self.changeiplabel.setText("设置局域网服务器ip地址（域名）")
        self.changeiplabel.setGeometry(90,230,200,24)
        self.ipedit.setFocus(True)
        self.ipedit.setGeometry(90,260,210,24)
        self.checkBtn.setText("确定")
        self.checkBtn.setEnabled(False)
        self.checkBtn.setGeometry(320,260,50,24)
        self.checkBtn.setStyleSheet("QPushButton{border-radius:4px;background-color:#d9d9d9;color:#444444;font-size:16px;}")
        # rx = QRegExp("\\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\b")
        # self.ipedit.setValidator(QRegExpValidator(rx))
        # self.ipedit.setInputMask("000.000.000.000;0")
        settings = QSettings("/etc/kum.conf", QSettings.IniFormat)
        settings.beginGroup("server_url")
        domain = settings.value("intranet")
        settings.endGroup()
        self.ipedit.setText(domain)
        # self.changeiplabel.hide()
        # self.ipedit.hide()
        # self.checkBtn.hide()

    #
    # 函数：绑定槽函数
    #
    def init_connect(self):
        self.checkBtn.clicked.connect(self.write_kum)
        self.ipedit.textChanged.connect(self.change_button_style)

    #
    # 函数：修改样式
    #
    def change_button_style(self):
        if self.change_tip == False:
            self.change_tip = True
        else:
            self.checkBtn.setEnabled(True)
            self.checkBtn.setStyleSheet(
                "QPushButton{border-radius:4px;background-color:#3d6be5;color:#ffffff;font-size:16px;}"
                "QPushButton:hover{border-radius:4px;background-color:#6b8eeb;color:#ffffff;font-size:16px;}"
                "QPushButton:pressed{border-radius:4px;background-color:#4e5fc4;color:#ffffff;font-size:16px;}")

    #
    # 函数：读取lineEdit数据，写入到kum配置文件中
    #
    def write_kum(self):
        self.checkBtn.setEnabled(False)
        self.checkBtn.setStyleSheet(
            "QPushButton{border-radius:4px;background-color:#d9d9d9;color:#444444;font-size:16px;}")
        self.parent.iface.call_change_internet_domain_name(self.ipedit.text())

    #
    # 函数：初始化按钮状态
    #
    def btvalue_init(self):
        cf = configparser.ConfigParser()
        cf.read(CONF_MODEL_PATH)
        bootauto = cf.get("Starting-mode","boot-auto")
        daily = cf.get("Starting-mode", "daily")
        #if(os.path.exists(AUTOSTART + "kylin-update-manager-autostart.desktop")):
        if(bootauto == "YES"):
            if (os.path.exists(AUTOSTART + "kylin-update-manager-autostart.desktop")):
                pass
            else:
                self.bootauto_check_file.emit(True)
            self.bootautocheckbt_initchecked = True

        else:
            self.bootautocheckbt_initchecked = False
        #else:
            #self.bootautocheckbt_initchecked = False
       # if(os.path.exists(CRONDAILY + "0kylin-update-manager-daily")):
        if(daily == "YES"):
            if (os.path.exists(CRONDAILY + "0kylin-update-manager-daily")):
                pass
            else:
                self.daily_check_file.emit(True)
            self.dailycheckbt_initchecked = True

        else:
            self.dailycheckbt_initchecked = False
        #else:
            #self.dailycheckbt_initchecked = False

    #
    # 函数：关闭按钮响应
    #
    def slot_click_close_button(self):
        self.hide()


    #
    # 函数：鼠标拖动窗口
    #
    def mousePressEvent(self, event):
        if(event.button() == Qt.LeftButton):
            self.mouse_press = True
            self.drag_pos = event.globalPos() - self.pos()
            event.accept()
    def mouseReleaseEvent(self, event):
        self.mouse_press = False
        self.setWindowOpacity(1)
    def mouseMoveEvent(self, event):
        if(self.mouse_press):
            self.move_pos = event.globalPos()
            self.move(self.move_pos - self.drag_pos)
            self.setWindowOpacity(1)
            event.accept()

    def paintEvent(self, event):
        # 阴影
        path = QPainterPath()
        path.setFillRule(Qt.WindingFill)
        pat = QPainter(self)
        pat.setRenderHint(pat.Antialiasing)
        # print(opt.palette.base().color())
        pat.fillPath(path, QBrush(Qt.white))

        color = QColor(0, 0, 0, 50)

        for i in range(10):

            i_path = QPainterPath()
            i_path.setFillRule(Qt.WindingFill)
            ref = QRectF(10 - i, 10 - i, self.width() - (10 - i) * 2, self.height() - (10 - i) * 2)
            if i == 0:
                pass
            else:
                i_path.addRoundedRect(ref, 0, 0)
                color.setAlpha(int(150 - i ** 0.5 * 50))
                pat.setPen(color)
                pat.drawPath(i_path)

        # 主题
        ref = QRectF(10, 10, self.width() - 20, self.height() - 20)
        p = QPainter(self)
        p.setRenderHint(QPainter.Antialiasing)
        p.setBrush(Qt.white)
        p.setPen(Qt.transparent)
        p.setPen(Qt.NoPen)
        # p.drawRect(opt.rect)
        p.drawRoundedRect(ref, 0, 0)

