#!/usr/bin/python3
# -*- coding: utf-8 -*-

__author__ = 'lixiang'

import os
import platform
import socket
import subprocess
import time
import urllib
from ftplib import FTP

import requests
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import QWidget, QCheckBox, QLabel, QToolTip, QProgressBar, QPushButton, QMessageBox

from utils.db_install_information import *
import re
from utils.enums import DOWN_CACHE_PATH, DOWN_TEMP_PATH

from mainui.updateitem import Ui_UpdateItem

import gettext
gettext.textdomain("kylin-update-manager")

_ = gettext.gettext


class LogListItemWidget(QWidget):

    app_msg = None
    size_change = pyqtSignal(str, bool)

    def __init__(self, appmsg, mw, is_log = False, parent=None):
        QWidget.__init__(self,parent)
        self.is_log = is_log
        #self.resize(550, 70)
        self.setFixedSize(550,70)
        #self.setGeometry(0,0,550,70)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.mw = mw
        self.app_msg = appmsg
        self.parent = parent
        self.ui_init()
        self.stylesheet_init()

    #
    # 函数：初始化ui
    #
    def ui_init(self):
        self.logolabel = QLabel(self)
        self.logolabel.setFixedSize(32, 32)
        self.logolabel.move(44,19)
        self.applabel = QLabel(self)
        self.applabel.setFixedWidth(240)
        self.applabel.move(90,15)
        self.appdescrible = QLabel(self)
        self.appdescrible.setFixedWidth(240)
        self.appdescrible.move(90,45)
        self.new_versionlabel = QLabel(self)
        self.new_versionlabel.setFixedWidth(125)
        self.new_versionlabel.move(355,27)
        self.installed_versionlabel = QLabel(self)
        self.installed_versionlabel.setFixedWidth(115)
        self.installed_versionlabel.move(300, 27)
        self.sizelabel = QLabel(self)
        self.sizelabel.setFixedWidth(50)
        self.sizelabel.move(480,27)
        self.speedlabel = QLabel(self)
        self.speedlabel.setFixedWidth(50)
        self.speedlabel.move(480, 27)
        self.speedlabel.hide()
        self.splitlabel = QLabel(self)
        self.splitlabel.setGeometry(0, 69, self.width(), 1)
        self.installed_time_label = QLabel(self)
        self.installed_time_label.setFixedSize(75,50)
        self.installed_time_label.move(475,17)
        self.installed_time_label.hide()
        self.install_statue = QLabel(self)
        self.install_statue.setFixedWidth(50)
        self.install_statue.move(420, 27)
        self.install_statue.hide()

    #
    # 函数：初始化样式
    #
    def stylesheet_init(self):
        pixmap = QPixmap(self.app_msg.ICON)
        if pixmap.isNull():
            pixmap = QPixmap("./res/soft_logo.png")
        self.logolabel.setPixmap(pixmap)
        self.logolabel.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.applabel.setStyleSheet("QLabel{font-size:16px;color:#444444;border:0px solid #cfcfcf;}")
        metrics = QFontMetrics(self.applabel.font())
        if (self.app_msg.NAME == "kylin-update-desktop-app"):
            elidedText = metrics.elidedText("基本应用", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-security"):
            elidedText = metrics.elidedText("安全更新", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-support"):
            elidedText = metrics.elidedText("系统基础组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-ukui"):
            elidedText = metrics.elidedText("桌面环境组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-kernel"):
            elidedText = metrics.elidedText("系统内核组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-kernel-3a4000"):
            elidedText = metrics.elidedText("系统内核组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-kydroid"):
            elidedText = metrics.elidedText("kydroid补丁包", Qt.ElideRight, self.applabel.width())
        else:
            elidedText = metrics.elidedText(self.app_msg.NAME, Qt.ElideRight, self.applabel.width())
        if elidedText.endswith("…"):
            #self.applabel.setToolTip(self.app_msg.NAME)
            self.applabel.setText(elidedText)
        else:
            self.applabel.setText(elidedText)

        self.setStyleSheet("QToolTip{border-radius:6px;border: 1px solid #888888;background:white;font-size:12px;color:#888888;width:200px;minimum=50px; maximum=300px;}")
        self.description  = self.get_app_description(self.app_msg)
        if(self.description == ""):
            if(self.app_msg.DESCRIPTION == ""):
                self.description = "changelog 开小差啦"
                self.judge_elidedText(self.appdescrible, "changelog 开小差啦")
            else:
                self.description = self.app_msg.DESCRIPTION
                self.judge_elidedText(self.appdescrible, self.app_msg.DESCRIPTION)
        else:
            self.judge_elidedText(self.appdescrible, self.description)
        self.appdescrible.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.sizelabel.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.sizelabel.setAlignment(Qt.AlignLeft)
        self.sizelabel.setText(self.app_msg.SIZE)
        self.speedlabel.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.speedlabel.setAlignment(Qt.AlignRight)
        self.splitlabel.setStyleSheet("QLabel{background-color:#eff0ef;border:0px solid #cfcfcf;}")
        #self.splitlabel2.setStyleSheet("QLabel{background-color:#cfcfcf;border:0px solid #cfcfcf;}")
        self.judge_elidedText(self.new_versionlabel, self.app_msg.CANDIDATEVERSION)
        self.new_versionlabel.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")
        self.installed_versionlabel.setText(_("installed version")+":"+self.app_msg.INSTALLEDVERSION)
        self.installed_versionlabel.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")
        self.install_statue.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")
        # self.connect(self.checkBox, SIGNAL("toggled(bool)"), self, SIGNAL("synCheckBoxState(bool)"))
        # self.checkBox.toggled.connect(self.slot_toggle_cb)
        # self.connect(self.checkBox, SIGNAL("toggled(bool)"), self, SLOT("synItemContext()"))
        self.installed_time_label.setAlignment(Qt.AlignLeft)
        self.installed_time_label.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")

    #
    # 函数：用于超过大小用...代替
    #
    def judge_elidedText(self, label, str):
        str = str.replace('\n', "")
        metrics = QFontMetrics(label.font())
        elidedText = metrics.elidedText(str, Qt.ElideRight, label.width())
        if elidedText.endswith("…"):
            # label.setToolTip(str)
            label.setText(elidedText)
        else:
            label.setText(elidedText)

    #
    #函数：用于超过大小用...代替
    #
    def judge_elidedText1(self, label, str):
        str = str.replace('\n'," ")
        metrics = QFontMetrics(label.font())
        elidedText = metrics.elidedText(str, Qt.ElideRight, label.width())
        str_len = len(str)
        num = int(str_len/20)
        num1 = str_len%20
        str1 = ""
        i = 0
        while(i<num):
            j = 0
            while j <20:
                str1 = str1 + str[i*20+j]
                j = j+1
            str1 = str1 + "\n"
            i = i+1
        k = 0
        while k < num1:
            str1 = str1 +str[num*20 + k]
            k = k+1
        if elidedText.endswith("…"):
            if(self.is_log == True):
                label.setToolTip(str1)
            label.setText(elidedText)
        else:
            label.setText(elidedText)


    #
    #函数：获取包的changelog到描述
    #
    def get_app_description(self, app_msg):
        try:
            description = ""
            if (self.kus.available_url.startswith("file") == False):
                cmd = "apt-get changelog " + app_msg + " > /dev/null 2>&1"
                result = os.popen(cmd).readlines()
                k = 0
                description = ""
                for j in result:
                    if (app_msg["version"] in j):
                        while (1):
                            version = re.findall(r'[(](.*?)[)]', result[k])
                            description = description + version[0] + "\n"
                            k = k + 2
                            while (1):
                                if (result[k] != "\n" or result[k] != " \n" or result[k] != "  \n"):
                                    result[k].strip("\n")
                                    description = description + result[k]
                                k = k + 1
                                if (("--" in result[k]) and ("<" in result[k]) and ("@" in result[k])):
                                    break
                            k = k + 2
                            #print(k)
                            if (k >= len(result)):
                                return description
                            if (app_msg["installversion"] in result[k]):
                                if (app_msg["installversion"] == "" or app_msg["installversion"] == " "):
                                    pass
                                else:
                                    version = re.findall(r'[(](.*?)[)]', result[k])
                                    description = description + version[0] + "\n"
                                    k = k + 2
                                    while (1):
                                        if (result[k] != "\n" or result[k] != " \n" or result[k] != "  \n"):
                                            result[k].strip("\n")
                                            description = description + result[k]
                                        k = k + 1
                                        if (("--" not in result[k]) and ("<" not in result[k]) and (
                                                "@" not in result[k])):
                                            break
                                    k = k + 2
                                    break
            return description
        except:
            return ""

    #
    # 函数：获取cb状态
    #
    def get_checkbox_status(self):
        return self.checkBox.isChecked()

    #
    # 函数：点击cb槽
    #
    def slot_toggle_cb(self):
        if self.checkBox.isChecked():
            self.size_change.emit(self.statusTip(), True)
        else:
            self.size_change.emit(self.statusTip(), False)

class UpdateListItemWidget(QWidget):
    download_progress_speed = pyqtSignal(str, int, int)  # 进度+速度信号
    install_success = pyqtSignal(str)  # 安装成功的信号
    size_change = pyqtSignal(str, bool)
    network_abnormal = pyqtSignal()
    check_all_statues = pyqtSignal(bool)
    copy_pks_signal = pyqtSignal(str)

    def __init__(self, appmsg, parent):
        QWidget.__init__(self,parent)
        self.is_working = False
        self.init_property()
        #self.resize(550, 70)
        self.setFixedSize(550,70)
        #self.setGeometry(0,0,550,70)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.app_msg = appmsg
        self.parent = parent
        self.ui_init()
        self.init_connect()
        if (platform.machine() == 'mips_64'):
            self.MIPS64 = True
        else:
            self.MIPS64 = False
        self.stylesheet_init()

    #
    # 函数：实例化实例属性
    #
    def init_property(self):
        self.app_msg = None
        self.prior_size = 0  # 下崽当前包之前的所有包的总大小
        self.down_size = 0
        self.all_size = 0
        self.need_copy_pkgs = []
        self.download_list = []
        self.is_working = False
        self.is_downloading = False
        self.connect_time = 0 #重连次数判断标志位


    #
    # 函数：初始化ui
    #
    def ui_init(self):
        self.download_progress = QProgressBar(self)
        self.download_progress.setMinimum(0)
        self.download_progress.setMaximum(100)
        self.download_progress.setTextVisible(False)
        self.download_progress.setValue(40)
        self.download_progress.setGeometry(0, 0, 550, 70)
        self.download_progress.hide()
        self.download_progress.setProperty("value", 0)
        self.install_progress = QProgressBar(self)
        self.install_progress.setMinimum(0)
        self.install_progress.setMaximum(100)
        self.install_progress.setTextVisible(False)
        self.install_progress.setValue(50)
        self.install_progress.setFixedSize(550, 70)
        self.install_progress.move(0, 0)
        self.install_progress.setProperty("value", 0)
        self.install_progress.hide()
        self.progress_mips = QProgressBar(self)
        self.progress_mips.setMinimum(0)
        self.progress_mips.setMaximum(100)
        self.progress_mips.setTextVisible(False)
        self.progress_mips.setValue(0)
        self.progress_mips.setFixedSize(380, 10)
        self.progress_mips.setGeometry(90, 50, 380, 10)
        self.progress_mips.hide()
        self.checkBox = QCheckBox(self)
        self.checkBox.move(15,27)
        self.logolabel = QLabel(self)
        self.logolabel.setFixedSize(32, 32)
        self.logolabel.move(44,19)
        self.applabel = QLabel(self)
        self.applabel.setFixedWidth(240)
        self.applabel.move(90,15)
        self.appdescrible = QLabel(self)
        self.appdescrible.setFixedWidth(240)
        self.appdescrible.move(90,45)
        self.new_versionlabel = QLabel(self)
        self.new_versionlabel.setFixedWidth(125)
        self.new_versionlabel.move(355,27)
        self.installed_versionlabel = QLabel(self)
        self.installed_versionlabel.setFixedWidth(115)
        self.installed_versionlabel.move(300, 27)
        self.sizelabel = QLabel(self)
        self.sizelabel.setFixedWidth(80)
        self.sizelabel.move(480,27)
        self.speedlabel = QLabel(self)
        self.speedlabel.setFixedWidth(50)
        self.speedlabel.move(400, 27)
        self.speedlabel.hide()
        self.progress_label = QLabel(self)
        self.progress_label.setFixedWidth(50)
        self.progress_label.move(355,27)
        # self.progress_label.setText("0.0%")
        self.progress_label.hide()
        self.tip_label = QLabel(self)
        self.tip_label.setFixedWidth(80)
        self.tip_label.move(355, 27)
        self.tip_label.hide()
        self.splitlabel = QLabel(self)
        self.splitlabel.setGeometry(0, 69, self.width(), 1)
        self.installed_time_label = QLabel(self)
        self.installed_time_label.setFixedSize(75,50)
        self.installed_time_label.move(475,17)
        self.installed_time_label.hide()
        self.install_statue = QLabel(self)
        self.install_statue.setFixedWidth(50)
        self.install_statue.move(420, 27)
        self.install_statue.hide()
        self.pause_btn = QPushButton(self)
        self.pause_btn.move(497, 27)
        self.pause_btn.setFixedSize(16,16)
        self.pause_btn.hide()
        self.pause_btn.setFocusPolicy(Qt.NoFocus)
        self.restart_btn = QPushButton(self)
        self.restart_btn.move(497, 27)
        self.restart_btn.setFixedSize(16,16)
        self.restart_btn.hide()
        self.restart_btn.setFocusPolicy(Qt.NoFocus)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.move(520, 27)
        self.cancel_btn.setFixedSize(16,16)
        self.cancel_btn.hide()
        self.cancel_btn.setFocusPolicy(Qt.NoFocus)
        self.set_pkg_size()

    #
    # 函数：连接信号槽
    #
    def init_connect(self):
        self.pause_btn.clicked.connect(self.pause_download)
        self.restart_btn.clicked.connect(self.restart_download)
        self.cancel_btn.clicked.connect(self.cancel_download)
        self.download_progress_speed.connect(self.parent.slot_set_progress_speed)
        self.install_success.connect(self.parent.remove_list)
        self.network_abnormal.connect(self.parent.show_network_abnormal)
        self.checkBox.stateChanged.connect(self.send_check_signal)
        self.check_all_statues.connect(self.parent.bottom_widget.check_important_status)
        self.copy_pks_signal.connect(self.parent.slot_copy_pkg_to_archive)

    #
    # 函数：点击勾选后发送检查信号
    #
    def send_check_signal(self, state):
        if state == Qt.Checked:
            self.check_all_statues.emit(True)
        else:
            self.check_all_statues.emit(False)

    #
    # 函数：初始化样式
    #
    def stylesheet_init(self):
        self.checkBox.setFocusPolicy(Qt.NoFocus)
        self.checkBox.setCheckState(Qt.Checked)
        self.checkBox.setStyleSheet("QCheckBox{border-radius:4px;spacing: 0px;color:#666666;font-size:14px;background-color:#FFFFFF;}" \
                                    "QCheckBox:hover{background-color:#DAE3FA;}" \
                                    "QCheckBox::indicator{width: 16px;height: 16px;border-radius:4px;}" \
                                    "QCheckBox::indicator:unchecked{background-color:#FFFFFF;border:1px solid #bfbfbf;}" \
                                    "QCheckBox::indicator:unchecked:hover{background-color:#DAE3FA;border:1px solid #6B8EEB;}" \
                                    "QCheckBox::indicator:unchecked:pressed{background-color:#BCCCF6;border:1px solid #415FC4;}" \
                                    "QCheckBox::indicator{width: 16px;height: 16px;}" \
                                    "QCheckBox::indicator:checked{background-color:#3D6BE5;border:1px solid #415FC4;image: url(./res/check);}" \
                                    "QCheckBox::indicator:checked:hover{background-color:#6B8EEB;border:1px solid #415FC4;image: url(./res/check);}" \
                                    "QCheckBox::indicator:checked:pressed{background-color:#415FC4;border:1px solid #415FC4;image: url(./res/check);}")
        if (self.MIPS64):
            self.download_progress.setStyleSheet("QProgressBar{border:1px solid #ffffff;background-color: rgba(0,0,0,0%);border-radius: 1px; }" \
                                      "QProgressBar {border: 0px solid grey;border-radius: 0px;text-align: center;}")
            self.install_progress.setStyleSheet("QProgressBar{border:1px solid #ffffff;background-color: rgba(0,0,0,0%);border-radius: 1px; }" \
                                        "QProgressBar {border: 0px solid grey;border-radius: 0px;text-align: center;}")
        else:
            self.download_progress.setStyleSheet("QProgressBar{border:1px solid #ffffff;background-color: rgba(0,0,0,0%);border-radius: 1px; }"\
                                        "QProgressBar::chunk {background-color: #dae3fa;border-radius:0px;}"\
                                        "QProgressBar {border: 0px solid grey;border-radius: 0px;text-align: center;}")
            self.install_progress.setStyleSheet("QProgressBar{border:1px solid #ffffff;background-color: rgba(0,0,0,0%);border-radius: 1px; }"\
                                        "QProgressBar::chunk {background-color: #bbcbf6;border-radius:0px;}"\
                                        "QProgressBar {border: 0px solid grey;border-radius: 0px;text-align: center;}")
        self.progress_mips.setStyleSheet("QProgressBar{background-color:#ffffff;border:0px solid #aaaaaa;border-radius:0px;color:#666666;text-align:center;}")
        pixmap = QPixmap(self.app_msg.ICON)
        if pixmap.isNull():
            pixmap = QPixmap("./res/soft_logo.png")
        self.logolabel.setPixmap(pixmap)
        self.logolabel.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
        self.applabel.setStyleSheet("QLabel{font-size:16px;color:#444444;border:0px solid #cfcfcf;}")
        metrics = QFontMetrics(self.applabel.font())
        if (self.app_msg.NAME == "kylin-update-desktop-app"):
            elidedText = metrics.elidedText("基本应用", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-security"):
            elidedText = metrics.elidedText("安全更新", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-support"):
            elidedText = metrics.elidedText("系统基础组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-ukui"):
            elidedText = metrics.elidedText("桌面环境组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-kernel"):
            elidedText = metrics.elidedText("系统内核组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-kernel-3a4000"):
            elidedText = metrics.elidedText("系统内核组件", Qt.ElideRight, self.applabel.width())
        elif (self.app_msg.NAME == "kylin-update-desktop-kydroid"):
            elidedText = metrics.elidedText("kydroid补丁包", Qt.ElideRight, self.applabel.width())
        else:
            elidedText = metrics.elidedText(self.app_msg.NAME, Qt.ElideRight, self.applabel.width())
        if elidedText.endswith("…"):
            #self.applabel.setToolTip(self.app_msg.NAME)
            self.applabel.setText(elidedText)
        else:
            self.applabel.setText(elidedText)

        self.setStyleSheet("QToolTip{border-radius:6px;border: 1px solid #888888;background:white;font-size:12px;color:#888888;width:200px;minimum=50px; maximum=300px;}")
        self.description  = self.get_app_description(self.app_msg)
        if(self.description == ""):
            if(self.app_msg.DESCRIPTION == ""):
                self.description = "changelog 开小差啦"
                self.judge_elidedText(self.appdescrible, "changelog 开小差啦")
            else:
                self.description = self.app_msg.DESCRIPTION
                self.judge_elidedText(self.appdescrible, self.app_msg.DESCRIPTION)
        else:
            self.judge_elidedText(self.appdescrible, self.description)
        self.appdescrible.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.sizelabel.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.sizelabel.setAlignment(Qt.AlignLeft)
        # self.sizelabel.setText(self.app_msg.SIZE)
        self.speedlabel.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.speedlabel.setAlignment(Qt.AlignRight)
        self.progress_label.setStyleSheet("QLabel{font-size:12px;color:#444444;}")
        self.progress_label.setAlignment(Qt.AlignLeft)
        self.splitlabel.setStyleSheet("QLabel{background-color:#eff0ef;border:0px solid #cfcfcf;}")
        #self.splitlabel2.setStyleSheet("QLabel{background-color:#cfcfcf;border:0px solid #cfcfcf;}")
        self.judge_elidedText(self.new_versionlabel, self.app_msg.CANDIDATEVERSION)
        self.new_versionlabel.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")
        self.installed_versionlabel.setText(_("installed version")+":"+self.app_msg.INSTALLEDVERSION)
        self.installed_versionlabel.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")
        self.install_statue.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")
        # self.connect(self.checkBox, SIGNAL("toggled(bool)"), self, SIGNAL("synCheckBoxState(bool)"))
        self.checkBox.toggled.connect(self.slot_toggle_cb)
        # self.connect(self.checkBox, SIGNAL("toggled(bool)"), self, SLOT("synItemContext()"))
        self.installed_time_label.setAlignment(Qt.AlignLeft)
        self.installed_time_label.setStyleSheet("QLabel{font-size:12px;color:#888888;border:0px solid #cfcfcf;}")
        self.pause_btn.setStyleSheet("QPushButton{background-color:#ffffff;border-radius:6px;image: url(./res/pause3);}"\
                                     "QPushButton:hover{background-color:#ffffff;border-radius:6px;image: url(./res/pause2);}"\
                                     "QPushButton:pressed{background-color:#ffffff;border-radius:6px;image: url(./res/pause1);}")
        self.restart_btn.setStyleSheet("QPushButton{background-color:#ffffff;border-radius:6px;image: url(./res/restart3);}"\
                                       "QPushButton:hover{background-color:#ffffff;border-radius:6px;image: url(./res/restart2);}"\
                                       "QPushButton:pressed{background-color:#ffffff;border-radius:6px;image: url(./res/restart1);}")
        self.cancel_btn.setStyleSheet("QPushButton{background-color:#ffffff;border-radius:6px;image: url(./res/cancel3);}"\
                                      "QPushButton:hover{background-color:#ffffff;border-radius:6px;image: url(./res/cancel2);}"\
                                      "QPushButton:pressed{background-color:#ffffff;border-radius:6px;image: url(./res/cancel1);}")
        self.tip_label.setStyleSheet("QLabel{font-size:12px;color:#444444;border:0px solid #cfcfcf;}")

    #
    # 函数：用于超过大小用...代替
    #
    def judge_elidedText(self, label, str):
        str = str.replace('\n', "")
        metrics = QFontMetrics(label.font())
        elidedText = metrics.elidedText(str, Qt.ElideRight, label.width())
        if elidedText.endswith("…"):
            # label.setToolTip(str)
            label.setText(elidedText)
        else:
            label.setText(elidedText)

    #
    #函数：用于超过大小用...代替
    #
    def judge_elidedText1(self, label, str):
        str = str.replace('\n'," ")
        metrics = QFontMetrics(label.font())
        elidedText = metrics.elidedText(str, Qt.ElideRight, label.width())
        str_len = len(str)
        num = int(str_len/20)
        num1 = str_len%20
        str1 = ""
        i = 0
        while(i<num):
            j = 0
            while j <20:
                str1 = str1 + str[i*20+j]
                j = j+1
            str1 = str1 + "\n"
            i = i+1
        k = 0
        while k < num1:
            str1 = str1 +str[num*20 + k]
            k = k+1
        if elidedText.endswith("…"):
            label.setText(elidedText)


    #
    #函数：获取包的changelog到描述
    #
    def get_app_description(self, app_msg):
        try:
            description = ""
            if (self.kus.available_url.startswith("file") == False):
                cmd = "apt-get changelog " + app_msg + " > /dev/null 2>&1"
                result = os.popen(cmd).readlines()
                k = 0
                description = ""
                for j in result:
                    if (app_msg["version"] in j):
                        while (1):
                            version = re.findall(r'[(](.*?)[)]', result[k])
                            description = description + version[0] + "\n"
                            k = k + 2
                            while (1):
                                if (result[k] != "\n" or result[k] != " \n" or result[k] != "  \n"):
                                    result[k].strip("\n")
                                    description = description + result[k]
                                k = k + 1
                                if (("--" in result[k]) and ("<" in result[k]) and ("@" in result[k])):
                                    break
                            k = k + 2
                            #print(k)
                            if (k >= len(result)):
                                return description
                            if (app_msg["installversion"] in result[k]):
                                if (app_msg["installversion"] == "" or app_msg["installversion"] == " "):
                                    pass
                                else:
                                    version = re.findall(r'[(](.*?)[)]', result[k])
                                    description = description + version[0] + "\n"
                                    k = k + 2
                                    while (1):
                                        if (result[k] != "\n" or result[k] != " \n" or result[k] != "  \n"):
                                            result[k].strip("\n")
                                            description = description + result[k]
                                        k = k + 1
                                        if (("--" not in result[k]) and ("<" not in result[k]) and (
                                                "@" not in result[k])):
                                            break
                                    k = k + 2
                                    break
            return description
        except:
            return ""

    #
    # 函数：获取cb状态
    #
    def get_checkbox_status(self):
        return self.checkBox.isChecked()

    #
    # 函数：点击cb槽
    #
    def slot_toggle_cb(self):
        if self.checkBox.isChecked():
            self.size_change.emit(self.statusTip(), True)
        else:
            self.size_change.emit(self.statusTip(), False)

    #
    # 函数：获取当前下载进度
    #
    def get_down_speed(self, appname, fullname, file_size):
        while(self.is_pause == False and self.is_cancel == False and self.parent.click_cancel_btn == False):
            pre_size = self.down_size
            cmd = "ls -l " + DOWN_CACHE_PATH + " |grep " + fullname + " |awk '{print $5}'"
            try:
                size = int(os.popen(cmd).readline())
            except:
                continue
            self.down_size = int(self.prior_size + size)
            speed = int(int(self.down_size - pre_size)/0.3)
            progress = int((self.down_size / self.all_size) * 100)
            # print("speed:", speed, "progress:", progress)
            self.download_progress_speed.emit(self.app_msg.NAME, speed, progress)
            # print("pre_size:", pre_size, "self.prior_size:", self.prior_size)
            # 重新连接成功后需要给标志位清零
            if speed != 0 and self.connect_time > 0:
                self.connect_time = 0
            if(int(size) == int(file_size)):
                self.download_list.append(appname) #添加已下载的包
                self.prior_size = self.prior_size + int(file_size)
                src = os.path.join(DOWN_CACHE_PATH, fullname)
                dst = DOWN_TEMP_PATH
                shutil.move(src, dst)
                self.need_copy_pkgs.append(os.path.join(DOWN_TEMP_PATH, fullname))
                break
            else:
                if self.download_process.poll() == 4 or self.download_process.poll() == 8:
                    # print("connect_time:", self.connect_time)
                    if self.is_pause == False:
                        # 每隔0.5秒重新连接20次
                        if self.connect_time == 20:
                            self.is_pause = True
                            self.connect_time = 0
                            self.network_abnormal.emit()
                            self.parent.worklist.clear() #工作列表清空
                            return True
                        else:
                            # print("connect again")
                            self.connect_time += 1
                            time.sleep(0.5)
                            return False
                elif self.download_process.poll() == 0:
                    self.download_list.append(appname)  # 添加已下载的包
                    self.prior_size = self.prior_size + int(file_size)
                    src = os.path.join(DOWN_CACHE_PATH, fullname)
                    dst = DOWN_TEMP_PATH
                    shutil.move(src, dst)
                    self.need_copy_pkgs.append(os.path.join(DOWN_TEMP_PATH, fullname))
                    break
            time.sleep(0.3)

    #
    # 函数：获取当前下载包的依赖
    #
    def get_deps_pkg(self, appname):
        app_msg = {"name":"","url":"","app_fullname":"","size":""}
        res = self.parent.iface.call_get_depends_pkgs(appname)
        # print(len(res))
        dep_list = []
        all_size = 0
        get_depends = False
        num = 0
        for i in res:
            if ("After this operation," in i):
                num = num + 1
                get_depends = True
                while (num < len(res)):
                    i = res[num]
                    # print(i)
                    i_list = i.strip("\n").split(" ")
                    app_msg["name"] = i_list[1].split("_")[0]
                    app_msg["size"] = i_list[2]
                    app_msg["url"] = i_list[0].strip("'")
                    app_msg["app_fullname"] = i_list[1]
                    dep_list.append(app_msg)
                    # self.need_copy_pkgs.append(i_list[1])
                    all_size = all_size + int(i_list[2])
                    app_msg = {"name": "", "app_fullname": "", "size": "", "url": ""}
                    num = num + 1
                break
            # 软件包已安装(kylin-update-desktop单独处理)-lj
            elif ("0 upgraded," in i):
                # print("len:",len(res), "num:", num+1)
                if len(res) == num + 1:
                    get_depends = True
                    break
            num = num + 1
        return dep_list, all_size, get_depends

    #
    # 函数：下载软件包
    #
    def down_deb_pkg(self):
        self.is_downloading = True
        for pkg in self.parent.App_Version_list:
            #未下载前点击了暂停，需要进程判断
            if pkg.NAME == self.app_msg.NAME and self.is_pause == False:
                dep_list, self.all_size, get_depeds = self.get_deps_pkg(self.app_msg.NAME)
                # print("allsize:", self.all_size)
                # print("dep_list:", dep_list)
                #检测到依赖有问题
                if get_depeds == False:
                    self.show_statue(_("Download dependency failed"), "fail")
                    # self.is_working = False
                    self.is_downloading = False
                    self.insert_database("Fail")
                    return
                else:
                    with open("/etc/apt/sources.list", "r") as file:
                        for line in file:
                            if "file://" in line:
                                self.cancel_btn.hide()
                                # self.restart_btn.hide()
                                self.pause_btn.hide()
                                self.show_statue(_("waiting for installing"), "success")
                                self.start_install(self.app_msg.NAME)
                                return
                    #此时软件包已经安装或者需要的包已经下载到了/var/apt/cache/archive目录下,可以直接安装-lj
                    if len(dep_list) == 0:
                        # print("包已存在直接安装")
                        self.start_install(self.app_msg.NAME)
                        return
                    log_file = os.path.join(DOWN_CACHE_PATH, "wget_log")
                    for i in dep_list:
                        if self.is_pause == True or self.is_cancel == True or self.parent.click_cancel_btn == True:
                            return
                        else:
                            file_deb_path = os.path.join(DOWN_CACHE_PATH, i["app_fullname"])
                            if i["name"] in self.download_list:
                                # print("in_dowload")
                                continue
                            #如果临时目录下已存在该软件包则直接跳过
                            if os.path.exists(os.path.join(DOWN_TEMP_PATH, i["app_fullname"]) or os.path.exists(os.path.join("/var/apt/cache/archive", i["app_fullname"]))):
                                self.down_size = self.down_size + int(i["size"])
                                self.prior_size = self.prior_size + int(i["size"])
                                self.download_list.append(i["name"]) #已下载列表(包名)
                                self.need_copy_pkgs.append(os.path.join(DOWN_TEMP_PATH, i["app_fullname"]))
                                continue
                            else:
                                # print("name:", i["name"], "fullname:", i["app_fullname"], "allsize:", self.all_size)
                                # self.th = self.parent.pool.submit(self.get_down_speed, i["name"], i["app_fullname"], i["size"], i["url"])
                                self.wget_and_ls(i, file_deb_path, log_file)
                    if self.is_pause == False and self.is_cancel == False and self.connect_time == 0:
                        # print("start copy all")
                        # 软件包已下载完开始拷贝包
                        self.is_downloading = False
                        self.copy_pks_signal.emit(self.app_msg.NAME)
                        # self.copy_pkg_to_archive()

    #
    # 函数：开启wget进程和ls线程
    #
    def wget_and_ls(self, i, file_deb_path,log_file):
        self.download_process = subprocess.Popen(["wget", "-c", "-P", DOWN_CACHE_PATH, i["url"], "-O", file_deb_path, "-o", log_file, "-T", "5"])
        self.th = self.parent.pool.submit(self.get_down_speed, i["name"], i["app_fullname"], i["size"])
        result = self.th.result()
        # print("result:",result)
        #根据result返回值判断是否需要进行重新连接
        if result == False:
            self.wget_and_ls(i, file_deb_path, log_file)
        else:
            return

    #
    # 函数：设置软件包大小
    #
    def set_pkg_size(self):
        dep_list, all_size, get_depends = self.get_deps_pkg(self.app_msg.NAME)
        if get_depends == True and all_size != 0:
            self.all_size = int(all_size)
        else:
            #解析依赖失败的时候
            self.all_size = int(str(self.app_msg.BYTESIZE).strip())
        if (self.all_size/1024) >= 1:
            if ((self.all_size/1024)/1024) >= 1:
                if (((self.all_size/1024)/1024)/1024) >= 1:
                    if ((((self.all_size/1024)/1024)/1024)/1024) >=1:
                        self.sizelabel.setText(str(round(self.all_size, 2))+"TB")
                    else:
                        self.sizelabel.setText(str(round((((self.all_size)/1024)/1024)/1024, 2))+"GB")
                else:
                    self.sizelabel.setText(str(round(((self.all_size)/1024)/1024 ,2))+"MB")
            else:
                self.sizelabel.setText(str(round((self.all_size)/1024, 2))+"KB")
        else:
            self.sizelabel.setText(str(self.all_size)+"B")
        self.sizelabel.show()

    #
    # 函数：设置下载速度和进度
    #
    def set_download_speed_progress(self, speed, progress):
        if self.is_pause == False and self.is_cancel == False:
            self.parent.set_tray_progress(self.app_msg.NAME, _("Download Progress: "), str(int(progress)))  # 设置右下角的下载速度
            self.install_progress.hide()
            self.progress_label.setText(str(progress) + "%")
            self.progress_label.show()
            self.download_progress.setValue(progress)
            self.download_progress.show()
            if speed != 0:
                if int(speed / 1024) >= 1:
                    if int(int(speed / 1024) / 1024) >= 1:
                        if int(int(int(speed / 1024) / 1024) / 1024) >= 1:
                            if int(int(int(int(speed / 1024) / 1024) / 1024) / 1024) >= 1:
                                self.speedlabel.setText(str(int((((speed / 1024) / 1024) / 1024) / 1024)) + "TB/S")
                            else:
                                self.speedlabel.setText(str(int(((speed / 1024) / 1024) / 1024)) + "GB/S")
                        else:
                            self.speedlabel.setText(str(int((speed / 1024) / 1024)) + "MB/S")
                    else:
                        self.speedlabel.setText(str(int(speed / 1024)) + "KB/S")
                else:
                    self.speedlabel.setText(str(speed) + "B/S")
            self.speedlabel.show()

    #
    # 函数：安装完成写入数据库
    #
    def insert_database(self, text):
        self.parent.get_app_changelog(self.app_msg, text)

    #
    # 函数：开始拷贝软件包到/var/cache/apt/archive目录下
    #
    def copy_pkg_to_archive(self):
        self.pause_btn.hide()
        self.cancel_btn.hide()
        self.speedlabel.hide()
        self.progress_label.hide()
        self.show_statue(_("waiting for installing"), "success")
        # print("name:", self.app_msg.NAME, "need_copy_file:", self.need_copy_pkgs)
        self.parent.iface.call_copy_deb(self.need_copy_pkgs, self.app_msg.NAME)

    #
    # 函数：切换到开始下载的界面
    #
    def switch_to_download(self):
        self.is_cancel = False
        self.is_pause = False
        self.checkBox.setEnabled(False)
        self.appdescrible.hide()
        self.new_versionlabel.hide()
        self.sizelabel.hide()
        self.speedlabel.show()
        self.progress_label.show()
        self.cancel_btn.show()
        self.restart_btn.hide()
        self.pause_btn.show()
        self.install_progress.show()

    #
    # 函数：接收暂停下载的信号
    #
    def slot_pause_download(self):
        if self.is_downloading:
            self.pause_download()

    #
    # 函数：暂停下载
    #
    def pause_download(self):
        self.connect_time = 0
        self.is_pause = True
        self.is_downloading = False
        self.pause_btn.setEnabled(False)
        self.checkBox.setEnabled(True)
        # print("pause download")
        try:
            if hasattr(self, "download_process"):
                self.download_process.terminate()
            self.is_working = False
            self.pause_btn.hide()
            self.restart_btn.show()
            self.pause_btn.setEnabled(True)
        except:
            print("download_process is not exists")

    #
    # 函数：重新下载
    #
    def restart_download(self):
        self.is_pause = False
        self.is_downloading = True
        self.is_working = True
        self.pause_btn.show()
        self.restart_btn.hide()
        self.checkBox.setEnabled(False)
        self.parent.mutex.acquire()
        self.parent.worklist.append(self)
        self.parent.mutex.release()


    #
    # 函数：取消下载
    #
    def cancel_download(self):
        # print("click cancel")
        self.connect_time = 0
        self.is_cancel = True
        self.is_working = False
        self.is_downloading = False
        # 此时没有正在下载，直接从任务列表中移除
        if self in self.parent.worklist:
            self.parent.worklist.remove(self)
            # print("remove success")
        else:
            try:
                if hasattr(self, "download_process"):
                    self.download_process.terminate()
            except:
                print("download_process is not exists")
            self.down_size = 0
            self.prior_size = 0
        if self.parent.is_SP_update == True: #sp升级时点击了取消按钮，恢复点击状态
            self.parent.bottom_widget.layout_all_cb.button_update.setEnabled(True)
            self.parent.bottom_widget.layout_all_cb.button_cancel.setEnabled(True)
            self.parent.bottom_widget.layout_all_cb.button_cancel.setStyleSheet(
                "QPushButton{border-radius:4px;background-color:#d9d9d9;color:#444444;font-size:16px;}"
                "QPushButton:hover{border-radius:4px;background-color:#6b8eeb;color:#ffffff;font-size:16px;}"
                "QPushButton:pressed{border-radius:4px;background-color:#4e5fc4;color:#ffffff;font-size:16px;}")
            self.parent.bottom_widget.layout_all_cb.button_update.setStyleSheet(
                "QPushButton{border-radius:4px;background-color:#3d6be5;color:#ffffff;font-size:16px;}"
                "QPushButton:hover{border-radius:4px;background-color:#6b8eeb;color:#ffffff;font-size:16px;}"
                "QPushButton:pressed{border-radius:4px;background-color:#4e5fc4;color:#ffffff;font-size:16px;}")
        self.download_progress.setValue(0)
        self.download_progress.hide()
        self.install_progress.hide()
        if self.app_msg.NAME != "kylin-update-desktop":
            self.appdescrible.show()
        self.new_versionlabel.show()
        self.speedlabel.hide()
        self.progress_label.hide()
        self.sizelabel.show()
        self.cancel_btn.hide()
        self.pause_btn.hide()
        self.restart_btn.hide()
        self.download_list.clear()
        self.need_copy_pkgs.clear()
        self.checkBox.setEnabled(True)

    #
    # 函数：开始下载with open
    #
    def start_down(self):
        self.is_cancel = False
        self.is_pause = False
        if self.parent.is_SP_update == True and self.app_msg.NAME == "kylin-update-desktop": #sp升级
            self.switch_to_download()
            self.parent.mutex.acquire()
            self.parent.worklist.append(self)
            self.parent.mutex.release()
            self.is_working = True #全局标识是否正在下载和安装
            self.checkBox.setEnabled(False)
            self.is_downloading = True
            return
        elif self.parent.is_SP_update == False and self.checkBox.isChecked() == True and self.is_working == False: #important升级
            # print("get signal")
            self.switch_to_download()
            self.parent.mutex.acquire()
            self.parent.worklist.append(self)
            self.parent.mutex.release()
            self.is_working = True
            self.checkBox.setEnabled(False)
            self.is_downloading = True


    #
    # 函数：开始安装
    #
    def start_install(self, appname):
        if appname == self.app_msg.NAME:
            # print("start_install")
            self.parent.important_worklist.append(self.app_msg.NAME)
            self.parent.iface.call_install_and_upgrade(appname)

    #
    # 函数：设置安装进度条
    #
    def set_install_progress(self, name, progress):
        if name == self.app_msg.NAME:
            self.show_statue(_("Installing..."), "success")
            self.install_progress.setValue(int(progress))
            # self.download_progress.hide()
            self.install_progress.show()
            if int(progress) == 100.0:
                self.show_statue(_("Installation successful"), "success")
                time.sleep(0.5)
                # print("start_move")
                # 发送移除listitem的信号
                self.install_success.emit(name)

    #
    # 函数：显示下载安装的状态
    #
    def show_statue(self, text, statue):
        self.tip_label.show()
        self.tip_label.setText(text)
        self.pause_btn.hide()
        self.cancel_btn.hide()
        if statue == "fail":
            self.tip_label.setStyleSheet("QLabel{font-size:12px;color:#d88888;border:0px solid #cfcfcf;}")

def main():
    update = UpdateListItemWidget()
    update.show()

if __name__ == '__main__':
    main()